function OscRecord=oscreadtbs(csvFileName)
  
# OscRecord = oscreadtbs(csvFileName)
#
# Reads data from a CSV file containg waveform data saved from a Tektronix 
# TBS series oscilloscope.
#
# The OscRecord is a strucure with the following elements:
#   - nWfms -- number of oscilloscope Waveforms (waveforms) for which data 
#       were found in the file
#   - Waveforms -- vector of structures containing data for the consecutive 
#       waveforms, with the following elements:
#       - Params -- structure containing waveform parameters; depending on the 
#           particular oscilloscope type, its members can be:
#           - length -- record length in samples
#           - dx -- sampling interval
#           - nTrigger -- trigger point index
#           - label -- waveform label as used by the oscilloscope
#           - yUnit -- vertical axis unit label
#           - yScale -- vertical scale (units per division) in the 
#               oscilloscopes screen (only informs about screen settings at the 
#               moment when data were saved, does not influence Y-values)
#           - yZero -- vertical offset of the waveform in the oscilloscope's 
#               screen (only informs about screen settings at the moment when 
#               data were saved, does not influence Y-values)
#           - yOffset -- vertical offset of the waveform (reflected in Y-values)
#           - xUnit -- horizontal axis unit label
#           - xScale -- horizontal scale (units per division) in the 
#               oscilloscopes screen (only informs about screen settings at 
#               the moment when data were saved, does not influence X-values)
#           - note - note assigned to the waveform
#       - Values -- structure containing waveform point coordinates, with the 
#           following elements:
#           - x -- vector of X-coordinates
#           - y -- vector of Y-coordinates

# Author:
#   Lukasz Starzak 
#   Dept. of Microelectronics and Computer Science, Lodz University of Technology

  NUMBER_COLUMNS = 6;
  COLUMN_PARAM_NAME = 1;
  COLUMN_X = 4;
  SIG_DIGITS = 4;
  MAX_EMPTY_ROWS = 5;

  [fCsv, msg] = fopen(csvFileName,"r");
  if (fCsv == -1)
    error(["Could not open file: " msg]);
  endif
  lineSplit = strsplit(fgetl(fCsv),",","collapsedelimiters",false);
  if (mod(size(lineSplit,2),NUMBER_COLUMNS) != 0)
    error("Incorrect input file format");
  endif
  nWfms = size(lineSplit,2)/NUMBER_COLUMNS;
  OscRecord.nWfms = nWfms;

  frewind(fCsv);
  nEmpty = 0;
  while (nEmpty <= MAX_EMPTY_ROWS)
    lineSplit = strsplit(fgetl(fCsv),",","collapsedelimiters",false);
    for iWfm=[1:nWfms]
      paramName = char(lineSplit((iWfm-1)*NUMBER_COLUMNS+COLUMN_PARAM_NAME));
      if (length(paramName) > 0)
        paramValue = char(lineSplit((iWfm-1)*NUMBER_COLUMNS+COLUMN_PARAM_NAME+1));
        switch (paramName)
          case "Record Length"
            OscRecord.Waveforms(iWfm).Params.length = str2num(paramValue);
          case "Sample Interval"
            OscRecord.Waveforms(iWfm).Params.dx = str2num(paramValue);
          case "Trigger Point"
            OscRecord.Waveforms(iWfm).Params.nTrigger = str2num(paramValue);
          case "Source"
            OscRecord.Waveforms(iWfm).Params.label = paramValue;
          case "Vertical Units"
            OscRecord.Waveforms(iWfm).Params.yUnit = paramValue;
          case "Vertical Scale"
            OscRecord.Waveforms(iWfm).Params.yScale = roundsigdgt(str2num(paramValue),SIG_DIGITS)*4/5;
          case "Vertical Offset"
            OscRecord.Waveforms(iWfm).Params.yOffset = roundsigdgt(str2num(paramValue),SIG_DIGITS);
          case "Horizontal Units"
            OscRecord.Waveforms(iWfm).Params.xUnit = paramValue;
          case "Horizontal Scale"
            OscRecord.Waveforms(iWfm).Params.xScale = roundsigdgt(str2num(paramValue),SIG_DIGITS);
          %case "Pt Fmt"
          case "Yzero"
            OscRecord.Waveforms(iWfm).Params.yZero = str2num(paramValue)*5/4;
          case "Note"
            OscRecord.Waveforms(iWfm).Params.note = paramValue;
        endswitch
        nEmpty = 0;
      else
        if (iWfm == 1)
          nEmpty = nEmpty+1;
        endif
      endif
    endfor
  endwhile

  fclose(fCsv);
    
  for iWfm=[1:nWfms]
      waveform = csvread(csvFileName, ...
          [0,(iWfm-1)*NUMBER_COLUMNS+COLUMN_X-1, ...
          OscRecord.Waveforms(iWfm).Params.length-1,(iWfm-1)*NUMBER_COLUMNS+COLUMN_X-1+1]);  
      OscRecord.Waveforms(iWfm).Values.x = waveform(:,1)';
      OscRecord.Waveforms(iWfm).Values.y = waveform(:,2)';
  endfor

endfunction
